/*
   Folia
   Copyright (C) 2007 Michal Vaner (michal.vaner@kdemail.net)

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   he Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the

   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
#include <QRegion>
#include <QApplication>
#include <QDesktopWidget>
#include <QPainter>
#include <QPen>
#include <QBrush>
#include <QBitmap>
#include <QColorDialog>
#include <QMouseEvent>
#include <QFileDialog>
#include <QRegExp>
#include <QCursor>
#include <QMenu>
#include <QTimer>
#include "draw_win.h"
#include "tput.h"

DrawWin::DrawWin() :
	draw( QApplication::desktop()->width(), QApplication::desktop()->height() ),
	color( 50, 50 ),
	label( 350, 10 ),
	actColor( 255, 0, 0 ),
	menu( new QMenu( this ) ),
	timer( new QTimer( this ) ),
	screenTimer( new QTimer( this ) ),
	activated( false ),
	buts( 0 )
{
	draw.fill( QColor( 0, 0, 0, 0 ) );
        setWindowFlags( windowFlags() | Qt::FramelessWindowHint /*| Qt::X11BypassWindowManagerHint*/ | Qt::WindowStaysOnTopHint );
        setGeometry( 0, 0, QApplication::desktop()->width(), QApplication::desktop()->height() );
	color.fill( QColor( 0, 0, 0, 0 ) );
	setColor( actColor );
	label.fill( QColor( 0, 0, 0, 0 ) );
	QPainter laPa;
	laPa.begin( &label );
	laPa.setPen( QPen( Qt::NoPen ) );
	laPa.setBrush( QBrush( Qt::black ) );
	QPoint points[] = { QPoint( 50, 0 ), QPoint( 60, 10 ), QPoint( 340, 10 ), QPoint( 350, 0 ) };
	laPa.drawConvexPolygon( points, 4 );
	laPa.end();
	constructMask();
	menu->addAction( tr( "&Clear" ), this, SLOT(clearDraw()) );
	menu->addSeparator();
	menu->addAction( tr( "&Save" ), this, SLOT(save()) );
	menu->addAction( tr( "&Load" ), this, SLOT(load()) );
	menu->addSeparator();
	menu->addAction( tr( "&Quit" ), QApplication::instance(), SLOT(quit()) );
        timer->setInterval( 0 );
	screenTimer->setInterval( 1000 );
	connect( timer, SIGNAL(timeout()), SLOT(doDraw()) );
	connect( screenTimer, SIGNAL(timeout()), SLOT(updateScreen()) );
	activ();
}

void DrawWin::setColor( const QColor &col ) {
	actColor = col;
	QPainter colPaint( &color );
	colPaint.setBrush( QBrush( actColor ) );
	colPaint.setPen( QPen( Qt::NoPen ) );
	colPaint.drawEllipse( 30, 30, 20, 20 );
	update();
}

void DrawWin::constructMask() {
	mask = QRegion( draw.mask() ).unite( QRegion( color.mask() ) ).unite( QRegion( label.mask() ) );
	if( activated )
		mask = mask.unite( QRegion( QCursor::pos().x() - 20, QCursor::pos().y() - 20, 40, 40, QRegion::Ellipse ) );
	setMask( mask );
}

void DrawWin::paintEvent( QPaintEvent *event ) {
	QPainter painter( this );
	painter.setClipRegion( QRegion( QCursor::pos().x() - 30, QCursor::pos().y() - 30, 60, 60, QRegion::Ellipse ) );
	painter.drawPixmap( 0, 0, screen );
	painter.setClipRegion( event->region() );
	painter.drawPixmap( 0, 0, label );
	painter.drawPixmap( 0, 0, draw );
	painter.drawPixmap( 0, 0, color );
}

void DrawWin::mousePressEvent( QMouseEvent *event ) {
	QWidget::mousePressEvent( event );
	bool used = false;
	switch( event->button() ) {
		case Qt::LeftButton:
			if( QRegion( color.mask() ).contains( event->pos() ) ) {
				bool wasActive = activated;
				if( activated )
					activ();
				QColor col = QColorDialog::getColor( actColor );
				if( col.isValid() )
					setColor( col );
				if( wasActive )
					activ();
				used = true;
			} else if( QRegion( label.mask() ).contains( event->pos() ) ) {
				activ();
				used = true;
			} else if( event->modifiers() & Qt::ShiftModifier ) {
				bool wasActive = activated;
				if( activated )
					activ();
				QString text(getTextInput());
				if(!text.isEmpty()) {
					QPainter painter;
					painter.begin(&draw);
					painter.setBrush(actColor);
					painter.setPen(actColor);
					QFont f = painter.font();
					f.setPixelSize(16);
					f.setBold(true);
					f.setStyleStrategy(QFont::NoAntialias);
					painter.setFont(f);
					painter.drawText(event->pos(), text);
					painter.end();
					constructMask();
					update();
				}
				if( wasActive )
					activ();
				used = true;
			}
			break;
		case Qt::RightButton:
			if( QRegion( label.mask() ).contains( event->pos() ) ) {
				bool wasActive = activated;
				if( activated )
					activ();
				menu->exec( event->globalPos() );
				used = true;
				if( wasActive )
					activ();
			}
			break;
		case Qt::MidButton:
			if( QRegion( label.mask() ).contains( event->pos() ) ) {
				QApplication::quit();
				used = true;
			} else
				activ();
			break;
		default:;
	}
	lastPos = event->pos();
	if( !used )
		buts = event->buttons();
}

void DrawWin::mouseReleaseEvent( QMouseEvent *event ) {
	QWidget::mouseReleaseEvent( event );
	buts = 0;
}

void DrawWin::doDraw() {
	if( buts & Qt::LeftButton ) {
		QPainter painter;
		painter.begin( &draw );
		QPen pen( actColor );
		pen.setWidth( 5 );
		pen.setCapStyle( Qt::RoundCap );
		painter.setPen( pen );
		painter.drawLine( lastPos, QCursor::pos() );
		painter.end();
		update( QRect( qMin( lastPos.x(), QCursor::pos().x() ) - 5, qMin( lastPos.y(), QCursor::pos().y() ) - 5, qAbs( lastPos.x() - QCursor::pos().x() ) + 10, qAbs( lastPos.y() - QCursor::pos().y() ) + 10 ) );
	} else if( buts & Qt::RightButton ) {
		QBitmap ms = draw.mask();
		QPainter painter;
		painter.begin( &ms );
		QPen pen( Qt::color0 );
		pen.setWidth( 30 );
		pen.setCapStyle( Qt::RoundCap );
		painter.setPen( pen );
		painter.drawLine( lastPos, QCursor::pos() );
		painter.end();
		draw.setMask( ms );
	}
	lastPos = QCursor::pos();
	constructMask();
}

void DrawWin::activ() {
	updateScreen();
	activated = !activated;
	if( activated ) {
		timer->start();
		screenTimer->start();
	} else {
		timer->stop();
		screenTimer->stop();
	}
	constructMask();
}

void DrawWin::updateScreen() {
	screen = QPixmap::grabWindow( QApplication::desktop()->winId() );
}

void DrawWin::clearDraw() {
	draw = QPixmap( QApplication::desktop()->width(), QApplication::desktop()->height() );
	draw.fill( QColor( 0, 0, 0, 0 ) );
	constructMask();
	update();
}

void DrawWin::save() {
	bool wasActive = activated;
	if( activated )
		activ();
	QString fn = QFileDialog::getSaveFileName( this, tr( "Select file name" ), QString(), "PNG images (*.png)" );
	if( !fn.isEmpty() ) {
		if( !QRegExp( ".*\\.png" ).exactMatch( fn ) )
			fn += ".png";
		draw.save( fn, "PNG" );
	}
	if( wasActive )
		activ();
}

void DrawWin::load() {
	bool wasActive = activated;
	if( activated )
		activ();
	QString fn = QFileDialog::getOpenFileName( this, tr( "Select the image" ), QString(), "PNG images (*.png)" );
	if( !fn.isEmpty() ) {
		draw = QPixmap( QApplication::desktop()->width(), QApplication::desktop()->height() );
		draw.fill( QColor( 0, 0, 0, 0 ) );
		QPixmap l( fn );
		QPainter pt;
		pt.begin( &draw );
		pt.drawPixmap( 0, 0, fn );
		pt.end();
		constructMask();
		update();
	}
	if( wasActive )
		activ();
}

#include "moc_draw_win.cpp"

